/*
 * taken from the Java Developers Journal...
 *
 */

import java.io.*;

public class UploadOutputStream extends OutputStream 
{
  
  private OutputStream internalOutputStream;
  private int length=-1;
  private long startTime=-1;
  private int bytesWriten=0;
  
  public UploadOutputStream(OutputStream os) 
  {
    internalOutputStream = os;
  }
  
  public UploadOutputStream(OutputStream os, int length) 
  {
    internalOutputStream = os;
    this.length = length;
  }
  
  public float getUploadRate() 
  {
    if (startTime != -1) 
      {
	long currentTime = System.currentTimeMillis();
	
	return bytesWriten / (float)(currentTime-startTime);
      }
    else
      return -1.0f;
  }
  
  public long getUploadTime() 
  {
      if (startTime != -1)
	return System.currentTimeMillis() - startTime;
      else
	return 0;
  }
  
  public float getUploadPercent() 
  {
    if (startTime == -1 || length == -1)
      return -1.0f;
    else 
      {
	return (float)bytesWriten / (float)length;
      }
  }
  
  public long getBytesWriten() 
  {
    return bytesWriten;
  }
  
  public long getBytesTotal() 
  {
    return length;
  }
  
  public long getEstimatedUploadTime() 
  {
    if (startTime == -1 || length == -1)
      return -1;
    
    long currentTime = System.currentTimeMillis();
    
    return (long)((length - bytesWriten) / getUploadRate());
  }
  
  public void close() throws IOException 
  {
    internalOutputStream.close();
  }
  
  public void flush() throws IOException 
  {
    internalOutputStream.flush();
  }
  
  public void write(byte[] b, int off, int len) throws IOException 
  {
    if (startTime == -1)
      startTime = System.currentTimeMillis();
    
    bytesWriten += len;
    internalOutputStream.write(b,off,len);
  }
  
   public void write(int b) throws IOException 
  {
    if (startTime == -1)
      startTime = System.currentTimeMillis();
    bytesWriten++;
    
    internalOutputStream.write(b);
  }
}
